@@ -17,7 +17,7 @@ def _separate_by_modes(
     '''        
     from ..utils import get_peaks 
     peaks = get_peaks( y )
-    if r is None: r = np.sqrt(len( peaks )) if len(peaks) >  10 else len(peaks)# uniform radius
+    if r is None: r = np.sqrt(len( peaks ))# uniform radius
     std = y.std()
     # means = np.zeros( [len(peaks)]  + list(X.shape[1:])  ) # in case dimensions are greater than 2
-    return [np.argwhere( (y >= p-(std/r)) & (p+(std/r) >= y) ) for i,p in enumerate(peaks) ], peaks
+    return [np.argwhere( (y <= p+(std/r)) & ( y >= (p-(std/r))) ) for i,p in enumerate(peaks) ], peaks
